package com.androidbook.achartengine.audiograph;

import org.achartengine.ChartFactory;
import org.achartengine.GraphicalView;
import org.achartengine.model.XYMultipleSeriesDataset;
import org.achartengine.model.XYSeries;
import org.achartengine.renderer.XYMultipleSeriesRenderer;
import org.achartengine.renderer.XYSeriesRenderer;

import android.app.Activity;
import android.graphics.Color;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.AudioRecord.OnRecordPositionUpdateListener;
import android.media.MediaRecorder;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.LinearLayout;

public class MainActivity extends Activity {

    static final String TAG = "AudioGraph";
    private LinearLayout mLayout = null;
    private boolean inRecordMode = false;
    private AudioRecord mAudioRecord;
    private int mAudioBufferSampleSize;
    private int mAudioBufferSize;
    private short[][] audioBuffer;
    private int[] bytesRead;

    private XYSeries audioSeries = new XYSeries("Audio");
    private int sampleRate = 8000;
    private int totalSampleSize = sampleRate * 3; // 3 seconds
    private int numBuffers = 3;
    private int procBuffer = 0;
    private double audioX = 0d;
    private int stepSize = 16; // we'll grab every 16th sample for plotting
    private double audioInc = ((double)stepSize) / ((double)sampleRate);

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        // Here's the layout to attach our chart to
        mLayout = (LinearLayout) findViewById(R.id.chart);
        // Setup for audio recording
        initAudioRecord();
    }

    // Our button calls this
    public void doClick(View view) {
        if(inRecordMode)
            return;
        inRecordMode = true;
        mLayout.removeAllViews(); // prepare to add a new chart
        Thread t = new Thread(new Runnable() {
            @Override
            public void run() {
                getSamples();
            }
        });
        t.start();
    }

    protected void onPause() {
        super.onPause();
        inRecordMode = false;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if(mAudioRecord != null) {
            mAudioRecord.release();
        }
    }

    // This listener will get called as audio is read in by getSamples()
    public OnRecordPositionUpdateListener mListener = new OnRecordPositionUpdateListener() {

        // This gets called every time we read a buffer
        public void onPeriodicNotification(AudioRecord recorder) {
            synchronized(audioSeries) {
                for(int j=0; j < bytesRead[procBuffer]; j+=stepSize) { // grab every step-th value
                    // Add to the XYSeries for later plotting
                    audioSeries.add(audioX, (double)audioBuffer[procBuffer][j]);
                    audioX += audioInc;
                }
                procBuffer = (procBuffer + 1) % numBuffers; // wrap around as needed
            }
        }

        public void onMarkerReached(AudioRecord recorder) {
            inRecordMode = false;
            Log.v(TAG, "stop recording");
            updateChart();
        }
    };

    private void initAudioRecord() {
        try {
            int channelConfig = AudioFormat.CHANNEL_IN_MONO;
            int audioFormat = AudioFormat.ENCODING_PCM_16BIT;
            mAudioBufferSize = 2 * AudioRecord.getMinBufferSize(sampleRate,
                    channelConfig, audioFormat);
            mAudioBufferSampleSize = mAudioBufferSize / 2;
            mAudioRecord = new AudioRecord(
                    MediaRecorder.AudioSource.MIC,
                    sampleRate,
                    channelConfig,
                    audioFormat,
                    mAudioBufferSize);
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        }

        mAudioRecord.setNotificationMarkerPosition(totalSampleSize);
        mAudioRecord.setPositionNotificationPeriod(mAudioBufferSampleSize);
        mAudioRecord.setRecordPositionUpdateListener(mListener);
    }

    // This runs in its own thread
    private void getSamples() {
        int i=0;
        procBuffer = 0; // procBuffer follows i
        // reset our data buffers
        audioSeries.clear();
        audioBuffer = new short[numBuffers][mAudioBufferSampleSize];
        bytesRead = new int[numBuffers];
        audioX = 0d;

        mAudioRecord.startRecording();

        while(inRecordMode) {
            bytesRead[i] = mAudioRecord.read(audioBuffer[i], 0, mAudioBufferSampleSize);
            i = (i + 1) % numBuffers; // wrap around while reading audio in
        }

        mAudioRecord.stop();
    }

    private void updateChart() {
        // We have a new waveform, so let's chart it.
        Log.v(TAG, "Making chart");
        XYMultipleSeriesDataset dataset = new XYMultipleSeriesDataset();
        dataset.addSeries(audioSeries);

        double minX = audioSeries.getMinX();
        double maxX = audioSeries.getMaxX();
        double minY = audioSeries.getMinY();
        double maxY = audioSeries.getMaxY();
        int itemCount = audioSeries.getItemCount();

        XYSeriesRenderer seriesRenderer = new XYSeriesRenderer();
        seriesRenderer.setColor(Color.MAGENTA);

        XYMultipleSeriesRenderer chartRenderer = new XYMultipleSeriesRenderer();
        // chartRenderer.setChartTitle("Recorded audio data");
        chartRenderer.setXAxisMin(0.0d);
        chartRenderer.setXAxisMax(maxX);
        chartRenderer.setYAxisMin(minY);
        chartRenderer.setYAxisMax(maxY);
        chartRenderer.setPanLimits(
                new double[] {0d, maxX*1.5, minY*1.2, maxY*1.2});
        chartRenderer.setAxisTitleTextSize(16);
        chartRenderer.setChartTitleTextSize(10);
        chartRenderer.setLabelsTextSize(15);
        chartRenderer.setLegendTextSize(15);

        chartRenderer.setMargins(new int[] {20, 30, 15, 0});
        chartRenderer.setXLabels(10);
        chartRenderer.setYLabels(10);
        chartRenderer.addSeriesRenderer(seriesRenderer);

        GraphicalView chartView = 
            ChartFactory.getLineChartView(this, dataset, chartRenderer);
        mLayout.addView(chartView,
            new LayoutParams(LayoutParams.FILL_PARENT,
                             LayoutParams.FILL_PARENT));
    }
}
